import 'dart:async';

import 'package_exception.dart';

mixin ErrorStream {
  final _errorController = StreamController<PackageException>();
  Stream<PackageException> get error => _errorController.stream;

  void handleError(void func()) {
    try {
      func();
    } catch (e, stackTrace) {
      recordError(e, stackTrace);
    }
  }

  void handleErrorAsync(Future future) =>
      future.catchError((e, stackTrace) => recordError(e, stackTrace));

  void recordError(e, [StackTrace stackTrace]) {
    if (e is Exception)
      _errorController.add(PackageException.fromException(e, stackTrace));
    else if (e is Error)
      _errorController.add(PackageException.fromError(e, stackTrace));
  }

  void disposeErrorStream() {
    _errorController.close();
  }
}
